<?php
/**
 * ownCloud - External Roundcube app
 *
 * @author      Zoltán Kovács <kzoli@kzoli.hu>
 * @license     GPL-3.0 http://www.gnu.org/licenses/gpl-3.0.html
 *
 * Based on the 'external' core app by Frank Karlitschek and on
 * the 3rd party 'roundcube' app by Martin Reinhardt and David Jaedke.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

namespace OCA\Roundcube_External;

/**
 * This class manages the roundcube_external app.
 *
 * @todo	Rewrite in real OOP style instead this procedural code.
 */
class Roundcube_External {

    /**
     * This static method tries to load the previously stored Roundcube
     * instances (if any) from the application's (JSON encoded) value.
     *
     * @brief 		Get a user's all Roundcube credentials.
     *
     * @return array	Two dimension array; rows are the Roundcube instances,
     *                  columns are the parameters for this instance. May empty.
     * [0..n]           actual Roundcube instance number (numeric key)
     *   [site_name]    arbitrary name, identifying this instance for users
     *   [site_url]     instance's URL [http(s)://fully.qualified.hostname/...]
     *   [site_icon]    icon for this instance in the Applications menu.
     *
     * @ todo		Use of \OCP\IConfig (interface) instead of \OCP\Config.
     * @ todo		Regexp check of data retrieved.
     */
    public static function getSites()
    {
	if (($sites = json_decode(\OCP\Config::getAppValue('roundcube_external', 'sites', ''))) != null)
	    return $sites;
	else
	    return array();
    }

    /**
     * This static method tries to save parameters of all available Roundcube instances
     * into a JSON-encoded value for the application.
     *
     * @brief 		Save parameters of all available Roundcube instances.
     *
     * @param array	Two dimension array; rows are the Roundcube instances,
     *                  columns are the parameters for this instance.
     * [0..n]           actual Roundcube instance number (numeric key)
     *   [site_name]    arbitrary name, identifying this instance for users
     *   [site_url]     instance's URL [http(s)://fully.qualified.hostname/...]
     *   [site_icon]    icon for this instance in the Applications menu.
     *
     * @ todo		Use of \OCP\IConfig (interface) instead of \OCP\Config.
     * @ todo		Regexp check of data stored.
     */
    public static function saveSites($sites = array())
    {
	if (sizeof($sites) == 0) { // removes value if no valid site
	    $appConfig = \OC::$server->getAppConfig();
	    $appConfig->deleteKey('roundcube_external', 'sites');
	    return true; }
	else { // saves settings
	    return \OCP\Config::setAppValue('roundcube_external', 'sites', json_encode($sites)); }
    }

    /**
     * This static method tries to load the previously stored Roundcube IMAP
     * credentials (if any) from the current ownCloud user's preferences.
     *
     * @brief 		Get a user's all Roundcube credentials.
     *
     * @return array	Two dimension array; rows are the Roundcube instances,
     *             	columns are the credentials for this instance. May empty.
     * [mail_host]	Roundcube hostname (unique)
     *  [mail_user]	IMAP username at this host
     *  [mail_password]	IMAP password (plain text)
     *
     * @ todo		Use of \OCP\IConfig (interface) instead of \OCP\Config.
     * @ todo		Regexp check of data retrieved.
     */
    public static function getCredentials()
    {
	// All credentials were stored as a two-dimensional, serialized array.
	$serialized = \OCP\Config::getUserValue(\OCP\User::getUser(), 'roundcube_external', 'credentials');
	$credentials = array();
	// If no stored credentials, nothing to do.
	if (empty($serialized)) return $credentials;

	// Check results - must be a serialized array.
	$unserialized = unserialize($serialized);
	if (!is_array($unserialized)) return $credentials;
	// Copy the necessary parameters only from the unserialized array.
	foreach (array_keys($unserialized) as $mail_host) {
	    // Each row key is a mail host; columns are the IMAP username and the plain-text (decoded) password.
	    $credentials[$mail_host] = array('mail_user' => $unserialized[$mail_host]['mail_user'],
	                                     'mail_password' => base64_decode($unserialized[$mail_host]['mail_password']));
	}
	return $credentials;
    }

    /**
     * This static method tries to save some Roundcube IMAP credentials
     * into preferences for the current ownCloud user.
     *
     * @brief 		Save all Roundcube credentials for a user.
     *
     * @param array	Two dimension array; rows are the Roundcube instances,
     *             	columns are the credentials for this instance. Checked.
     * [mail_host]	Roundcube hostname (unique)
     *  [mail_user]	IMAP username at this host
     *  [mail_password]	IMAP password (plain text)
     *
     * @return bool	true on success, false on failure.
     *
     * @ todo		Use of \OCP\IConfig (interface) instead of \OCP\Config.
     * @ todo		Regexp check of data to save.
     */
    public static function saveCredentials($credentials = array())
    {
	if (empty($credentials)) return true; // nothing to do
	foreach (array_keys($credentials) as $mail_host) {
            // Each row key is a mail host; columns are the IMAP username and the plain-text password.
            // The password will be stored with week encoding, against accidental prying eyes only :)
	    if (!empty($credentials[$mail_host]['mail_password'])) {
		$credentials[$mail_host]['mail_password'] = base64_encode($credentials[$mail_host]['mail_password']);
	    }
	}
	return \OCP\Config::setUserValue(\OCP\User::getUser(), 'roundcube_external', 'credentials', serialize($credentials));
    }

}
